<?php

/**
 * @file Workflow-ng integration for the token module
 * It exposes some further tokens as well as token enabled conditions and actions 
 */

/**
 * Implementation of hook_token_values()
 */
function workflow_ng_token_values($entity_type, $object = NULL) {
  $values = array();
  if ($entity_type == 'global') {
    $values['url'] = check_plain($_REQUEST['q']);
    $values['path'] = check_plain($_GET['q']);
    $arguments = explode('/', $values['path']); 
    foreach ($arguments as $key => $arg) {
      $values['path-arg-'. ($key+1)] = $arg;
    }
  }
  return $values;
}

/**
 * Implementation of hook_token_list()
 */
function workflow_ng_token_list($entity_type = 'all') {
  $tokens = array();
  $tokens['global']['url'] = t('The current active url, e.g. "admin".');
  $tokens['global']['path'] = t('The current active drupal path, e.g. "node/2". If URL aliases are used, this will be always the path behind the alias."');
  $tokens['global']['path-arg-1'] = t('The first component of the current active path, e.g. for the path "node/2" it will be "node".');
  $tokens['global']['path-arg-x'] = t('The Xst argument of the current active path.');
  return $tokens;
}






/*
 * Implementation of hook_condition_info()
 */
function token_condition_info() {
  return array(
    'workflow_ng_condition_token_compare' => array(
      '#label' => t('Textual comparison'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#description' => t('This condition just compares two texts. It returns TRUE, if both texts are equal.').
        ' '. t('You may use token replacements in both texts, e.g. the texts %text1 and %text2 are only equal, if the argument called "node" is of the type page.', array('%text1' => "[node:type]", '%text2' => 'page')),
      '#module' => t('Token'),
    ),
    'workflow_ng_condition_token_numeric' => array(
      '#label' => t('Numeric comparison'),
      '#arguments' => array(),
      '#description' => t('This does a numeric comparison between two tokens. Select greater than, less than or equal to.'),
      '#module' => t('Token'),
    ),
  );
}

/*
 * Condition Implementation: Token enabled comparison 
 */
function workflow_ng_condition_token_compare($settings, &$arguments, &$log) {
  $text1 = workflow_ng_token_replace($settings['text1'], $settings['used arguments1'], $arguments, $log);
  $text2 = workflow_ng_token_replace($settings['text2'], $settings['used arguments2'], $arguments, $log);
  return $settings['regex'] ? ereg($text2, $text1) : $text1 == $text2;
}

/*
 * Condition: Token enabled comparison configuration form
 */
function workflow_ng_condition_token_compare_form($settings = array(), $argument_info) {
  $form = array();
  $form['text1'] = array(
    '#type' => 'textfield',
    '#title' => 'Two texts to compare',
    '#default_value' => isset($settings['text1']) ? $settings['text1'] : '',
  );
  $form['text2'] = array(
    '#type' => 'textfield',
    '#title' => '',
    '#default_value' => isset($settings['text2']) ? $settings['text2'] : '',
  );
  $form['regex'] = array(
    '#type' => 'checkbox',
    '#title' => t('Evaluate text2 as a regular expression.'),
    '#default_value' => isset($settings['regex']) ? $settings['regex'] : 0,
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_condition_token_compare_submit($form_id, $form_values) {
  //returns the needed settings
  $settings = array('regex' => $form_values['regex']);
  foreach (array(1, 2) as $i) {
    $settings['used arguments'. $i] = workflow_ng_token_get_used_arguments($form_values['text'. $i], $form_values['arguments']);
    $settings['text'. $i] = $form_values['text'. $i];
  }
  return $settings;
}

/*
 * Condition implementation: token enabled numeric comparison
 */
function workflow_ng_condition_token_numeric($settings, &$arguments, &$log) {
  $text1 = (float)workflow_ng_token_replace($settings['text1'], $settings['used arguments1'], $arguments, $log);
  $text2 = (float)workflow_ng_token_replace($settings['text2'], $settings['used arguments2'], $arguments, $log);

  switch ($settings['operation']) {
    case 'greater':
      $result = $text1 > $text2;
      break;
    case 'equal':
      $result = $text1 == $text2;
      break;
    case 'less':
      $result = $text1 < $text2;
      break;
  }
  return $result;
}

/*
 * Condition: token enabled numeric comparison configuration form
 */
function workflow_ng_condition_token_numeric_form($settings = array(), $argument_info) {
  $form = array();
  $form['text1'] = array(
    '#type' => 'textfield',
    '#title' => t('Text to compare'),
    '#default_value' => isset($settings['text1']) ? $settings['text1'] : '',
  );
  $form['operation'] = array(
    '#type' => 'select',
    '#title' => t('Operation'),
    '#options' => array('greater' => t('Greater than'), 'equal' => t('Equal to'), 'less' => t('Less than')),
    '#default_value' => $settings['operation'],
  );
  $form['text2'] = array(
    '#type' => 'textfield',
    '#title' => '',
    '#default_value' => isset($settings['text2']) ? $settings['text2'] : '',
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_condition_token_numeric_submit($form_id, $form_values) {
  $settings = array('operation' => $form_values['operation']);
  foreach (array(1, 2) as $i) {
    $settings['used arguments'. $i] = workflow_ng_token_get_used_arguments($form_values['text'. $i], $form_values['arguments']);
    $settings['text'. $i] = $form_values['text'. $i];
  }
  return $settings;
}







/*
 * Implementation of hook_action_info()
 */
function token_action_info() {
  return array(
    'workflow_ng_action_mail_to_user' => array(
      '#label' => t('Send a mail to a user'),
      '#arguments' => array(
        'user' => array('#entity' => 'user', '#label' => t('Recipient')),
      ),
      '#module' => t('Token'),
      '#description' => t('You may use the token replacements for the mail\'s message body as well as the mail\'s subject.'),
    ),
    'workflow_ng_action_mail' => array(
      '#label' => t('Send a mail to an arbitrary mail address'),
      '#module' => t('Token'),
      '#description' => t('You may use the token replacements for the mail\'s message body, to and from fields as well as for the mail\'s subject.'),
    ),
    'workflow_ng_action_drupal_message' => array(
      '#label' => t('Show a configurable message on the site'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('Token'),
    ),
    'workflow_ng_action_drupal_goto' => array(
      '#label' => t('Page redirect'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('Token'),
    ),
    'workflow_ng_action_watchdog' => array(
      '#label' => t('Log to watchdog'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('Token'),
    ),
  );
}

/*
 * Action Implementation: Show a configureable message 
 */
function workflow_ng_action_drupal_message($settings, &$arguments, &$log) {
  $message = workflow_ng_token_replace($settings['message'], $settings['used arguments'], $arguments, $log);
  drupal_set_message($message, $settings['error'] ? 'error' : 'status');
}

/*
 * Action "Show a configureable message" configuration form
 */
function workflow_ng_action_drupal_message_form($settings = array(), $argument_info) {
  $form = array();
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The message that should be displayed.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  $form['error'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display as error message'),
    '#default_value' => $settings['error'],
  );
  return $form;
}

function workflow_ng_action_drupal_message_submit($form_id, $form_values) {
  //returns the needed settings
  $used_args = workflow_ng_token_get_used_arguments($form_values['message'], $form_values['arguments']);
  return array('message' => $form_values['message'], 'used arguments' => $used_args, 'error' => $form_values['error']);
}

/*
 * Action Implementation: Page redirect
 */
function workflow_ng_action_drupal_goto($settings, &$arguments, &$log) {
  $path = workflow_ng_token_replace($settings['path'], $settings['path_args'], $arguments, $log);
  $query = workflow_ng_token_replace($settings['query'], $settings['path_args'], $arguments, $log);
  $fragment = workflow_ng_token_replace($settings['fragment'], $settings['path_args'], $arguments, $log);
  if ($settings['force']) {
    if (strpos($query, 'destination') === FALSE) {
      if (!empty($query)) {
        $query .= '&';
      }
      $query .= $_REQUEST['destination'] ? 'destination='. urlencode($_REQUEST['destination']) : NULL;
    }
    unset($_REQUEST['destination']);
  }
  drupal_goto($path, $query ? $query : NULL, $fragment ? $fragment : NULL);
}

/*
 * Action "Page redirect" configuration form
 */
function workflow_ng_action_drupal_goto_form($settings = array(), $argument_info) {
  $form['redirect'] = array(
    '#type' => 'item',
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
    '#title' => t('To'),
    '#description' => '<div style="display:block">'. t('Enter a Drupal path, path alias, or external URL to redirect to. Enter (optional) queries after "?" and (optional) anchor after "#". Most redirects will not contain queries or fragment anchors.') .'</div>',
  );
  $form['redirect']['path'] = array(
    '#type' => 'textfield',
    '#size' => 30,
    '#default_value' => drupal_get_path_alias($settings['path']),
  );
  $form['redirect'][] = array(
    '#value' => '?',
  );
  $form['redirect']['query'] = array(
    '#type' => 'textfield',
    '#size' => 12,
    '#default_value' => $settings['query'],
  );
  $form['redirect'][] = array(
    '#value' => '#',
  );
  $form['redirect']['fragment'] = array(
    '#type' => 'textfield',
    '#size' => 12,
    '#default_value' => $settings['fragment'],
  );
  $form['force'] = array(
    '#type' => 'checkbox',
    '#title' => t('Force redirecting to the given path, even if a destination paramter is given'),
    '#description' => t('Per default drupal doesn\'t redirect to the given path, if a destination parameter is set.'.
      'Instead it redirects to the given destination paramter. Most times, the destination parameter is set by appending it to the URL, e.g. !example_url', array('!example_url' => 'http://example.com/user/login?destination=node/2')),
    '#default_value' => $settings['force'],
    '#weight' => 7,
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_drupal_goto_submit($form_id, $form_values) {
  //returns the needed settings
  $used_args = workflow_ng_token_get_used_arguments($form_values['path'] . $form_values['query'] . $form_values['fragment'] , $form_values['arguments']);
  return array('path' => $form_values['path'], 'query' => $form_values['query'], 'fragment' => $form_values['fragment'], 'force' => $form_values['force'], 'path_args' => $used_args);
}

/*
 * Action Implementation: Watchdog
 */
function workflow_ng_action_watchdog($settings, &$arguments, &$log) {
  $message = workflow_ng_token_replace($settings['message'], $settings['message_args'], $arguments, $log);
  $link = workflow_ng_token_replace($settings['link'], $settings['link_args'], $arguments, $log);
  watchdog($settings['type'], $message, $settings['severity'], $link);
}

/*
 * Action "Watchdog" configuration form
 */
function workflow_ng_action_watchdog_form($settings = array(), $argument_info) {
  $form['severity'] = array(
    '#type' => 'select',
    '#title' => t('Severity'),
    '#options' => array(
      WATCHDOG_NOTICE => t('Notice'),
      WATCHDOG_WARNING => t('Warning'),
      WATCHDOG_ERROR => t('Error'),
    ),
    '#default_value' => isset($settings['type']) ? $settings['type'] : WATCHDOG_NOTICE,
    '#required' => TRUE,
  );
  $form['type'] = array(
    '#type' => 'textfield',
    '#title' => t('Category'),
    '#default_value' => isset($settings['type']) ? $settings['type'] : 'workflow-ng',
    '#description' => t('The category to which this message belongs.'),
    '#required' => TRUE,
  );
  $form['message'] = array(
    '#type' => 'textfield',
    '#title' => t('Message'),
    '#default_value' => isset($settings['message']) ? $settings['message'] : '',
    '#description' => t('The message to log. You may make use of the replacements.'),
    '#required' => TRUE,
  );
  $form['link'] = array(
    '#type' => 'textfield',
    '#title' => t('Link (optional)'),
    '#default_value' => isset($settings['link']) ? $settings['link'] : '',
    '#description' => t('A link to associate with the message. You may make use of the replacements.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_watchdog_submit($form_id, $form_values) {
  $vars = drupal_map_assoc(array('severity', 'type', 'message', 'link'));
  $settings = array('message_args' => workflow_ng_token_get_used_arguments($form_values['message'], $form_values['arguments']));
  $settings['link_args'] = workflow_ng_token_get_used_arguments($form_values['link'], $form_values['arguments']);
  return array_intersect_key($form_values, $vars) + $settings;
}

/*
 * Action Implementation: Send a mail to a user
 */
function workflow_ng_action_mail_to_user($user, $settings, &$arguments, &$log) {
  //we also handle $settings['to'] if it's set
  foreach (array('from', 'subject', 'to', 'message') as $key) {
    if (isset($settings[$key])) {
      $$key = workflow_ng_token_replace($settings[$key], $settings[$key .'_args'], $arguments, $log);
    }
    if ($key != 'message' && isset($$key)) {
      $$key = str_replace(array("\r", "\n"), '', $$key);
    }
  }
  $to = isset($to) ? $to : $user->mail;

  if (drupal_mail('workflow_ng_action_mail', $to, $subject, $message, $from)) {
    watchdog('action', t('Sent email to %recipient', array('%recipient' => ($to == $user->mail) ? $user->name : $to)), WATCHDOG_NOTICE, ($to == $user->mail) ? theme('username', $user) : NULL);
  }
  else {
    watchdog('error', t('Unable to send email to %recipient', array('%recipient' => $user->name)), WATCHDOG_NOTICE, theme('username', $user));
  }
}

/*
 * Action "Send a mail to a user" configuration form
 */
function workflow_ng_action_mail_to_user_form($settings = array(), $argument_info) {
  $form['from'] = array(
    '#type' => 'textfield',
    '#title' => t('Sender'),
    '#default_value' => isset($settings['from']) ? $settings['from'] : variable_get('site_mail', ini_get('sendmail_from')),
    '#description' => t('The mail\'s "from" address.'),
    '#required' => TRUE,
  );
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => $settings['subject'],
    '#description' => t('The mail\'s subject.'),
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The mail\'s message body.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_mail_to_user_submit($form_id, $form_values) {
  $settings = array();
  foreach (array('from', 'subject', 'message') as $key) {
    $settings[$key] = $form_values[$key];
    $settings[$key .'_args'] = workflow_ng_token_get_used_arguments($form_values[$key], $form_values['arguments']);
  }
  return $settings;
}

/*
 * Action Implementation: workflow_ng_action_mail
 * This action makes use of the workflow_ng_action_mail_to_user action implementation
 */
function workflow_ng_action_mail($settings, &$arguments, &$log) {
  workflow_ng_action_mail_to_user(NULL, $settings, $arguments, $log);
}

/*
 * Action "Send a mail to an arbitrary mail address" configuration form
 */
function workflow_ng_action_mail_form($settings = array(), $argument_info) {
  $form = workflow_ng_action_mail_to_user_form($settings, $argument_info);
  $form['to'] = array(
    '#type' => 'textfield',
    '#title' => t('Recipient'),
    '#default_value' => $settings['to'],
    '#description' => t('The mail\'s recipient address. You may separate multiple addresses with ",".'),
    '#required' => TRUE,
    '#weight' => -1,
  );
  return $form;
}

function workflow_ng_action_mail_submit($form_id, $form_values) {
  $settings = workflow_ng_action_mail_to_user_submit($form_id, $form_values);
  $settings['to'] = $form_values['to'];
  $settings['to_args'] = workflow_ng_token_get_used_arguments($form_values['to'], $form_values['arguments']);
  return $settings;
}



/*
 * Some useful funtions that conditions and actions may use when token is enabled 
 */


/*
 * Returns an array of all used arguments in $text
 * 
 * @param $text The $text, which will be processed
 * @param $argument_info The argument definitions we work with
 * @return The array of names of the used arguments
 */
function workflow_ng_token_get_used_arguments($text, $argument_info) {
  $used_args = array();
  foreach ($argument_info as $name => $argument) {
    if (strpos($text, '['. $name . ':') !== FALSE) {
      $used_args[] = $name;
    }
  }
  return $used_args;
}

/*
 * Performs the replacement of the replacement patterns.
 * If a used argument isn't loaded yet, it will be automatically loaded by workflow_ng_element_get_argument()
 * 
 * @param $text The $text, which will be processed
 * @param $used_arguments The array of names of used arguments
 * @param $arguments The workflow-ng arguments data structure
 * @param $log An array of log entries. Set it to FALSE to disable logging
 * @return The text with replaced patterns.
 */
function workflow_ng_token_replace($text, $used_arguments, &$arguments, &$log) {
  static $token_cache = array();

  foreach($used_arguments as $name) {
    $data = workflow_ng_element_get_argument($arguments, $name, $log);
    $entity = $arguments['info'][$name]['#entity'];

    $token_id = _token_get_id($entity, $data);
    if (isset($token_cache[$token_id]) && $token_cache[$token_id] != $name) {
      //this is another argument although it has the same id
      //so we need to flush the token cache manually 
      token_get_values('global', NULL, TRUE);
    }

    $text = token_replace($text, $entity, $data, '['. $name . ':', ']');

    //remember that this argument has been used and token has cached it...
    $token_cache[$token_id] = $name;
  }
  return $text;
}

/*
 * Adds the token replacement help to $form
 */
function workflow_ng_token_replacement_help(&$form, $argument_info) {
  if (isset($argument_info) && count($argument_info)) {
    $form['replace'] = array(
      '#type' => 'fieldset',
      '#title' => t('Replacement patterns'),
      '#description' => t('You can make use of the replacement patterns of all available arguments.'),
    );
    foreach ($argument_info as $name => $argument) {
      $form['replace'][$name] = array(
        '#type' => 'fieldset',
        '#title' => t('Replacement patterns for @name', array('@name' => t($argument['#label']))),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );
      $form['replace'][$name]['content'] = array(
        '#value' => theme('token_help', $argument['#entity'], '['. $name . ':', ']'),
      );
    }
  }
  $form['arguments'] = array('#type' => 'value', '#value' => $argument_info);
}
